@@ -844,12 +844,12 @@ class AbstractFileSystem(up, metaclass=_Cached):
             self.put_file(lpath, rpath, **kwargs)
 
     def head(self, path, size=1024):
-        """ Get the first ``size`` bytes from file """
+        """Get the first ``size`` bytes from file"""
         with self.open(path, "rb") as f:
             return f.read(size)
 
     def tail(self, path, size=1024):
-        """ Get the last ``size`` bytes from file """
+        """Get the last ``size`` bytes from file"""
         with self.open(path, "rb") as f:
             f.seek(max(-size, -f.size), 2)
             return f.read()
@@ -911,7 +911,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
         return list(sorted(out))
 
     def mv(self, path1, path2, recursive=False, maxdepth=None, **kwargs):
-        """ Move file(s) from one location to another """
+        """Move file(s) from one location to another"""
         self.copy(path1, path2, recursive=recursive, maxdepth=maxdepth)
         self.rm(path1, recursive=recursive)
 
@@ -972,7 +972,15 @@ class AbstractFileSystem(up, metaclass=_Cached):
             **kwargs,
         )
 
-    def open(self, path, mode="rb", block_size=None, cache_options=None, **kwargs):
+    def open(
+        self,
+        path,
+        mode="rb",
+        block_size=None,
+        cache_options=None,
+        compression=None,
+        **kwargs,
+    ):
         """
         Return a file-like object from the filesystem
 
@@ -989,6 +997,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
             Some indication of buffering - this is a value in bytes
         cache_options : dict, optional
             Extra arguments to pass through to the cache.
+        compression: string or None
+            If given, open file using compression codec. Can either be a compression
+            name (a key in ``fsspec.compression.compr``) or "infer" to guess the
+            compression from the filename suffix.
         encoding, errors, newline: passed on to TextIOWrapper for text mode
         """
         import io
@@ -1003,7 +1015,15 @@ class AbstractFileSystem(up, metaclass=_Cached):
                 if k in kwargs
             }
             return io.TextIOWrapper(
-                self.open(path, mode, block_size, **kwargs), **text_kwargs
+                self.open(
+                    path,
+                    mode,
+                    block_size=block_size,
+                    cache_options=cache_options,
+                    compression=compression,
+                    **kwargs,
+                ),
+                **text_kwargs,
             )
         else:
             ac = kwargs.pop("autocommit", not self._intrans)
@@ -1015,6 +1035,14 @@ class AbstractFileSystem(up, metaclass=_Cached):
                 cache_options=cache_options,
                 **kwargs,
             )
+            if compression is not None:
+                from fsspec.compression import compr
+                from fsspec.core import get_compression
+
+                compression = get_compression(path, compression)
+                compress = compr[compression]
+                f = compress(f, mode=mode[0])
+
             if not ac and "r" not in mode:
                 self.transaction.files.append(f)
             return f
@@ -1393,14 +1421,14 @@ class AbstractBufferedFile(io.IOBase):
         """Throw away temporary file"""
 
     def info(self):
-        """ File information about this path """
+        """File information about this path"""
         if "r" in self.mode:
             return self.details
         else:
             raise ValueError("Info not available while writing")
 
     def tell(self):
-        """ Current file location """
+        """Current file location"""
         return self.loc
 
     def seek(self, loc, whence=0):
@@ -1507,7 +1535,7 @@ class AbstractBufferedFile(io.IOBase):
         # may not yet have been initialized, may need to call _initialize_upload
 
     def _initiate_upload(self):
-        """ Create remote file/upload """
+        """Create remote file/upload"""
         pass
 
     def _fetch_range(self, start, end):
